@@ -87,12 +87,39 @@ static QColor ratioToColor(float ratio)
 	return ratio > 0.8 ? green : Qt::red;
 }
 
-
-
-KTorrentViewItem::KTorrentViewItem(QListView* parent,TorrentInterface* tc)
-	: KListViewItem(parent),tc(tc)
+static double Percentage(const TorrentStats & s)
 {
-	m_parent = (KTorrentView*) parent;
+	if (s.bytes_left_to_download == 0)
+	{
+		return 100.0;
+	}
+	else
+	{
+		if (s.total_bytes_to_download == 0)
+		{
+			return 100.0;
+		}
+		else
+		{
+			double perc = 100.0 - ((double)s.bytes_left_to_download / s.total_bytes_to_download) * 100.0;
+			if (perc > 100.0)
+				perc = 100.0;
+			else if (perc > 99.9)
+				perc = 99.9;
+			else if (perc < 0.0)
+				perc = 0.0;
+			
+			return perc;
+		}
+	}
+}
+
+
+
+KTorrentViewItem::KTorrentViewItem(KTorrentView* parent,TorrentInterface* tc)
+	: KListViewItem(parent->listView()),tc(tc)
+{
+	m_parent = parent;
 	update();
 }
 
@@ -100,6 +127,46 @@ KTorrentViewItem::KTorrentViewItem(QListView* parent,TorrentInterface* tc)
 KTorrentViewItem::~KTorrentViewItem()
 {}
 
+QCStringList KTorrentViewItem::getTorrentInfo(kt::TorrentInterface* tc)
+{
+	QCStringList info;
+	const TorrentStats & s = tc->getStats();
+	info.append(s.torrent_name.local8Bit());
+	info.append(tc->statusToString().local8Bit());
+	info.append(BytesToString(s.bytes_downloaded).local8Bit());
+	info.append(BytesToString(s.total_bytes_to_download).local8Bit());
+	info.append(BytesToString(s.bytes_uploaded).local8Bit());
+	if (s.bytes_left_to_download == 0)
+		info.append(KBytesPerSecToString(0).local8Bit());
+	else
+		info.append(KBytesPerSecToString(s.download_rate / 1024.0).local8Bit());
+	
+	info.append(KBytesPerSecToString(s.upload_rate / 1024.0).local8Bit());
+	if (s.bytes_left_to_download == 0)
+	{
+		info.append(QCString(""));
+	}
+	else if (s.running) 
+	{
+		Uint32 secs = tc->getETA();
+		if(secs == -1)
+			info.append(i18n("infinity").local8Bit());
+		else
+			info.append(DurationToString(secs).local8Bit());
+	}
+	else
+	{
+		info.append(i18n("infinity").local8Bit());
+	}
+	
+	info.append(QString::number(s.num_peers).local8Bit());
+	info.append(QString(KGlobal::locale()->formatNumber(Percentage(s),2) + " %").local8Bit());
+	info.append(KGlobal::locale()->formatNumber(kt::ShareRatio(s),2).local8Bit());
+	info.append(QString::number(s.seeders_connected_to).local8Bit());
+	info.append(QString::number(s.leechers_connected_to).local8Bit());
+	return info;
+}
+
 void KTorrentViewItem::update()
 {
 	const TorrentStats & s = tc->getStats();
@@ -124,15 +191,25 @@ void KTorrentViewItem::update()
 	
 	if(m_parent->columnVisible(5))
 	{
-		if (s.bytes_left_to_download == 0)
-			setText(5,KBytesPerSecToString(0));
+		if (s.download_rate >= 103) // lowest "visible" speed, all below will be 0,0 Kb/s
+		{
+			if (s.bytes_left_to_download == 0)
+				setText(5,KBytesPerSecToString(0));
+			else
+				setText(5,KBytesPerSecToString(s.download_rate / 1024.0));
+		}
 		else
-			setText(5,KBytesPerSecToString(s.download_rate / 1024.0));
+			setText(5, "");
 	}
-	
+
 	if(m_parent->columnVisible(6))
-		setText(6,KBytesPerSecToString(s.upload_rate / 1024.0));
-  
+	{
+		if (s.upload_rate >= 103) // lowest "visible" speed, all below will be 0,0 Kb/s
+			setText(6,KBytesPerSecToString(s.upload_rate / 1024.0));
+		else
+			setText(6, "");
+	}
+
 	if(m_parent->columnVisible(7))
 	{
 		if (s.bytes_left_to_download == 0)
@@ -162,40 +239,38 @@ void KTorrentViewItem::update()
 	}
 	if(m_parent->columnVisible(8))	
 		setText(8,QString::number(s.num_peers));
+	
+	if(m_parent->columnVisible(8))
+	{
+		setText(8,QString("%1 (%2)").arg(QString::number(s.seeders_connected_to)).arg(QString::number(s.seeders_total)));	
+	}
 
 	if(m_parent->columnVisible(9))
 	{
-		double perc = 0;
-		if (s.bytes_left_to_download == 0)
-		{
-			perc = 100.0;
-		}
-		else
-		{
-			if (s.total_bytes_to_download == 0)
-			{
-				perc = 100.0;
-			}
-			else
-			{
-				perc = 100.0 - ((double)s.bytes_left_to_download / s.total_bytes_to_download) * 100.0;
-				if (perc > 100.0)
-					perc = 100.0;
-				else if (perc > 99.9)
-					perc = 99.9;
-				else if (perc < 0.0)
-					perc = 0.0;
-					
-			}
-		}
-		
-		setText(9,i18n("%1 %").arg(KGlobal::locale()->formatNumber(perc,2)));
+		setText(9,QString("%1 (%2)").arg(QString::number(s.leechers_connected_to)).arg(QString::number(s.leechers_total)));
 	}
-	
+
 	if(m_parent->columnVisible(10))
+	{
+		setText(10,i18n("%1 %").arg(KGlobal::locale()->formatNumber(Percentage(s),2)));
+	}
+	
+	if(m_parent->columnVisible(11))
 	{
 		float ratio = kt::ShareRatio(s);
-		setText(10,QString("%1").arg(KGlobal::locale()->formatNumber(ratio,2)));
+		setText(11,QString("%1").arg(KGlobal::locale()->formatNumber(ratio,2)));
+	}
+	
+	if (m_parent->columnVisible(12))
+	{
+		Uint32 secs = tc->getRunningTimeDL();
+		setText(12,secs > 0 ? DurationToString(secs) : "");
+	}
+	
+	if (m_parent->columnVisible(13))
+	{
+		Uint32 secs = tc->getRunningTimeUL() - tc->getRunningTimeDL();
+		setText(13,secs > 0 ? DurationToString(secs) : "");
 	}
 }
 
@@ -231,8 +306,9 @@ int KTorrentViewItem::compare(QListViewItem * i,int col,bool) const
 				return -1;
 			else
 				return CompareVal(eta,other->eta);
-		case 8: return CompareVal(s.num_peers,os.num_peers);
-		case 9:
+		case 8: return CompareVal(s.seeders_total,os.seeders_total);
+		case 9: return CompareVal(s.leechers_total,os.leechers_total);
+		case 10:
 		{
 			double perc = s.total_bytes_to_download == 0 ? 100.0 : ((double)s.bytes_downloaded / s.total_bytes_to_download) * 100.0;
 			if (perc > 100.0)
@@ -242,12 +318,20 @@ int KTorrentViewItem::compare(QListViewItem * i,int col,bool) const
 				operc = 100.0;
 			return CompareVal(perc,operc);
 		}
-		case 10:
+		case 11:
 		{
 			float r1 = kt::ShareRatio(s);
 			float r2 = kt::ShareRatio(os);
 			return CompareVal(r1,r2);
 		}
+		case 12:
+			return CompareVal(tc->getRunningTimeDL(),otc->getRunningTimeDL());
+		case 13:
+		{
+			Uint32 t = tc->getRunningTimeUL() - tc->getRunningTimeDL();
+			Uint32 ot = otc->getRunningTimeUL() - otc->getRunningTimeDL();
+			return CompareVal(t,ot);
+		}
 	}
 
 	return 0;
@@ -262,7 +346,7 @@ void KTorrentViewItem::paintCell(QPainter* p,const QColorGroup & cg,
 	if (column == 1)
 		_cg.setColor(QColorGroup::Text, StatusToColor(tc->getStats().status,cg));
 	
-	if (column == 10)
+	if (column == 11)
 		_cg.setColor(QColorGroup::Text, ratioToColor(kt::ShareRatio(tc->getStats())));
 
 
