@@ -21,6 +21,7 @@
 #define KTORRENTVIEWITEM_H
 
 #include <klistview.h>
+#include <dcopclient.h>
 #include <util/constants.h>
 
 namespace kt
@@ -38,11 +39,13 @@ class KTorrentViewItem : public KListViewItem
 	kt::TorrentInterface* tc;
 	bt::Int64 eta;
 public:
-	KTorrentViewItem(QListView* parent,kt::TorrentInterface* tc);
+	KTorrentViewItem(KTorrentView* parent,kt::TorrentInterface* tc);
 	virtual ~KTorrentViewItem();
 
 	kt::TorrentInterface* getTC() {return tc;}
 	void update();
+	
+	static QCStringList getTorrentInfo(kt::TorrentInterface* tc);
 
 private:
 	int compare(QListViewItem * i,int col,bool ascending) const;
