@@ -56,7 +56,24 @@ namespace kt
 		ALLOCATING_DISKSPACE,
 		ERROR,
 		QUEUED,
-		CHECKING_DATA
+		CHECKING_DATA,
+  		NO_SPACE_LEFT
+	};
+	
+	enum TorrentStartResponse
+	{
+		START_OK,
+		USER_CANCELED,
+		NOT_ENOUGH_DISKSPACE, 
+		MAX_SHARE_RATIO_REACHED, 
+		BUSY_WITH_DATA_CHECK, 
+		QM_LIMITS_REACHED // Max seeds or downloads reached
+	};
+	
+	enum AutoStopReason
+	{
+		MAX_RATIO_REACHED,
+		MAX_SEED_TIME_REACHED
 	};
 
 	struct TorrentStats
@@ -89,6 +106,8 @@ namespace kt
 		Uint32 num_chunks_downloaded;
 		/// Get the number of chunks which have been excluded
 		Uint32 num_chunks_excluded;
+		/// Get the number of chunks left
+		Uint32 num_chunks_left;
 		/// Size of each chunk
 		Uint32 chunk_size;
 		/// Total seeders in swarm
@@ -131,6 +150,8 @@ namespace kt
 		bool user_controlled;
 		/// Maximum share ratio
 		float max_share_ratio;
+		/// Maximum seed time
+		float max_seed_time;
 		/// Private torrent (i.e. no use of DHT)
 		bool priv_torrent;
 		/// Number of corrupted chunks found since the last check
@@ -232,6 +253,14 @@ namespace kt
 		 * @return true upon succes
 		 */
 		virtual bool changeDataDir(const QString & new_dir) = 0;
+		
+		/**
+		 * Change torrents output directory. If this fails we will fall back on the old directory.
+		 * @param new_dir The new directory
+		 * @param moveFiles Wheather to actually move the files or just change the directory without moving them.
+		 * @return true upon success.
+		 */
+		virtual bool changeOutputDir(const QString& new_dir, bool moveFiles = true) = 0;
 
 		/**
 		 * Roll back the previous changeDataDir call.
@@ -287,10 +316,18 @@ namespace kt
 		///Set the torrent queue number.
 		virtual void setPriority(int p) = 0;
 		
+		/// Set the max share ratio
 		virtual void setMaxShareRatio(float ratio) = 0;
 		
+		/// Get the max share ratio
 		virtual float getMaxShareRatio() const = 0;
 		
+		/// Set the max seed time in hours (0 is no limit)
+		virtual void setMaxSeedTime(float hours) = 0;
+		
+		/// Get the max seed time
+		virtual float getMaxSeedTime() const = 0;
+		
 		/// Make a string of the current status
 		virtual QString statusToString() const = 0;
 		
@@ -353,6 +390,9 @@ namespace kt
 		///Checks if a seeding torrent has reached its maximum share ratio
 		virtual bool overMaxRatio() = 0;
 		
+		/// Checks if a seeding torrent has reached it's max seed time
+		virtual bool overMaxSeedTime() = 0;
+		
 		/// Handle an error
 		virtual void handleError(const QString & err) = 0;
 		
@@ -380,6 +420,17 @@ namespace kt
 		/// Get our PeerID
 		virtual const bt::PeerID & getOwnPeerID() const = 0;
 		
+		/// Set the traffic limits for this torrent
+		virtual void setTrafficLimits(Uint32 up,Uint32 down) = 0;
+		
+		/// Get the traffic limits
+		virtual void getTrafficLimits(Uint32 & up,Uint32 & down) = 0;
+		
+		/// Check if there is enough diskspace available for this torrent
+		virtual bool checkDiskSpace(bool emit_sig = true) = 0;
+		
+		/// Are we in the process of moving files
+		virtual bool isMovingFiles() const = 0;
 	signals:
 		/**
 		 * Emited when we have finished downloading.
@@ -405,7 +456,7 @@ namespace kt
 		 * Happens when torrent has reached maximum share ratio and maybe we'll add something more...
 		 * @param me The object which emitted the signal.
 		 */
-		void seedingAutoStopped(kt::TorrentInterface* me);
+		void seedingAutoStopped(kt::TorrentInterface* me,kt::AutoStopReason reason);
 		
 		/**
 		 * Emitted just before the torrent is started, this should be used to do some
@@ -427,6 +478,19 @@ namespace kt
 		 * @param me The torrent which emitted the signal
 		 */
 		void corruptedDataFound(kt::TorrentInterface* me);
+		
+		/**
+		 * Disk is running out of space.
+		 * @param me The torrent which emitted the signal
+		 * @param toStop should this torrent be stopped or not
+		 */
+		void diskSpaceLow(kt::TorrentInterface* me, bool toStop);
+		
+		/**
+		 * Torrent has been stopped
+		 * @param me The torrent which emitted the signal
+		 */
+		void torrentStopped(kt::TorrentInterface* me);
 
 	protected:
 		TorrentStats stats;
