@@ -30,7 +30,7 @@
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 
-#ifdef Q_OS_LINUX
+#if defined(Q_OS_LINUX) && !defined(__FreeBSD_kernel__)
 #include <asm/ioctls.h>
 #endif
 
@@ -38,7 +38,7 @@
 #include <sys/filio.h>
 #endif
 
-#if defined(Q_OS_MACX) || defined(Q_OS_DARWIN) || (defined(Q_OS_FREEBSD) && __FreeBSD_version < 600020) || defined (Q_OS_SOLARIS) || defined(Q_OS_NETBSD) 
+#ifndef MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
 #endif
 
@@ -56,13 +56,14 @@ namespace net
 
 	Socket::Socket(int fd) : m_fd(fd),m_state(IDLE)
 	{
-#if defined(Q_OS_MACX) || defined(Q_OS_DARWIN) || (defined(Q_OS_FREEBSD) && __FreeBSD_version < 600020)
+#if defined(Q_OS_MACX) || defined(Q_OS_DARWIN) || (defined(Q_OS_FREEBSD) && !defined(__DragonFly__) && __FreeBSD_version < 600020)
 		int val = 1; 
 		if (setsockopt(m_fd,SOL_SOCKET,SO_NOSIGPIPE,&val,sizeof(int)) < 0)
 		{
 			Out(SYS_CON|LOG_NOTICE) << QString("Failed to set the NOSIGPIPE option : %1").arg(strerror(errno)) << endl;
 		}
 #endif
+		cacheAddress();
 	}
 	
 	Socket::Socket(bool tcp) : m_fd(-1),m_state(IDLE)
@@ -73,7 +74,7 @@ namespace net
 			Out(SYS_GEN|LOG_IMPORTANT) << QString("Cannot create socket : %1").arg(strerror(errno)) << endl;
 		}
 		m_fd = fd;
-#if defined(Q_OS_MACX) || defined(Q_OS_DARWIN) || (defined(Q_OS_FREEBSD) && __FreeBSD_version < 600020)
+#if defined(Q_OS_MACX) || defined(Q_OS_DARWIN) || (defined(Q_OS_FREEBSD) && !defined(__DragonFly__) && __FreeBSD_version < 600020)
 		int val = 1;
 		if (setsockopt(m_fd,SOL_SOCKET,SO_NOSIGPIPE,&val,sizeof(int)) < 0)
 		{
@@ -85,13 +86,17 @@ namespace net
 	Socket::~Socket()
 	{
 		if (m_fd >= 0)
+		{
+			shutdown(m_fd, SHUT_RDWR);
 			::close(m_fd);
+		}
 	}
 	
 	void Socket::close()
 	{
 		if (m_fd >= 0)
 		{
+			shutdown(m_fd, SHUT_RDWR);
 			::close(m_fd);
 			m_fd = -1;
 			m_state = CLOSED;
@@ -127,6 +132,7 @@ namespace net
 			}
 		}
 		m_state = CONNECTED;
+		cacheAddress();
 		return true;
 	}
 	
@@ -256,17 +262,17 @@ namespace net
 		return sfd;
 	}
 	
-	bool Socket::setTOS(char type_of_service)
+	bool Socket::setTOS(unsigned char type_of_service)
 	{
-#if defined(Q_OS_MACX) || defined(Q_OS_DARWIN) || (defined(Q_OS_FREEBSD) && __FreeBSD_version < 600020) || defined(Q_OS_NETBSD) || defined(Q_OS_BSD4)
-		int c = type_of_service;
+#if defined(Q_OS_MACX) || defined(Q_OS_DARWIN) || (defined(Q_OS_FREEBSD) && __FreeBSD_version < 600020) || defined(Q_OS_NETBSD) || defined(Q_OS_OPENBSD) || defined(Q_OS_BSD4)
+		unsigned int c = type_of_service;
 #else
-		char c = type_of_service;
+		unsigned char c = type_of_service;
 #endif
 		if (setsockopt(m_fd,IPPROTO_IP,IP_TOS,&c,sizeof(c)) < 0)
 		{
 			Out(SYS_CON|LOG_NOTICE) << QString("Failed to set TOS to %1 : %2")
-					.arg((int)type_of_service).arg(strerror(errno)) << endl;
+					.arg(type_of_service).arg(strerror(errno)) << endl;
 			return false;
 		}
 		return true;
@@ -281,20 +287,6 @@ namespace net
 		return ret;
 	}
 	
-	Address Socket::getPeerName() const
-	{
-		struct sockaddr_in addr;
-		socklen_t slen = sizeof(struct sockaddr_in);
-		if (getpeername(m_fd,(struct sockaddr*)&addr,&slen) == 0)
-		{
-			return Address(inet_ntoa(addr.sin_addr),ntohs(addr.sin_port));
-		}
-		else
-		{
-			return Address();
-		}
-	}
-	
 	bool Socket::connectSuccesFull()
 	{
 		if (m_state != CONNECTING)
@@ -306,10 +298,21 @@ namespace net
 			return false;
 		
 		if (err == 0)
+		{
 			m_state = CONNECTED;
+			cacheAddress();
+		}
 		
 		return err == 0;
 	}
+	
+	void Socket::cacheAddress()
+	{
+		struct sockaddr_in raddr;
+		socklen_t slen = sizeof(struct sockaddr_in);
+		if (getpeername(m_fd,(struct sockaddr*)&raddr,&slen) == 0)
+			addr = Address(inet_ntoa(raddr.sin_addr),ntohs(raddr.sin_port));
+	}
 
 	/*
 	void Socket::setReadBufferSize(int rbs)
