@@ -23,6 +23,7 @@
 #include <util/log.h>
 #include <util/error.h>
 #include <util/fileops.h>
+#include <util/waitjob.h>
 #include <torrent/globals.h>
 #include <interfaces/guiinterface.h>
 #include "pluginmanager.h"
@@ -38,8 +39,8 @@ namespace kt
 		unloaded.setAutoDelete(false);
 		plugins.setAutoDelete(false);
 		prefpage = 0;
-		pltoload.append("infowidgetplugin");
-		pltoload.append("searchplugin");
+		pltoload.append("Info Widget");
+		pltoload.append("Search");
 	}
 
 	PluginManager::~PluginManager()
@@ -51,11 +52,6 @@ namespace kt
 
 	void PluginManager::loadPluginList()
 	{
-		if (!prefpage)
-		{
-			prefpage = new PluginManagerPrefPage(this);
-			gui->addPrefPage(prefpage);
-		}
 		KTrader::OfferList offers = KTrader::self()->query("KTorrent/Plugin");
 
 		KTrader::OfferList::ConstIterator iter;
@@ -87,6 +83,13 @@ namespace kt
 			if (pltoload.contains(plugin->getName()))
 				load(plugin->getName());
 		}
+		
+		if (!prefpage)
+		{
+			prefpage = new PluginManagerPrefPage(this);
+			gui->addPrefPage(prefpage);
+		}
+		prefpage->updatePluginList();
 	}
 
 
@@ -114,6 +117,22 @@ namespace kt
 		Plugin* p = plugins.find(name);
 		if (!p)
 			return;
+		
+		// first shut it down properly
+		bt::WaitJob* wjob = new WaitJob(2000);
+		try
+		{
+			p->shutdown(wjob);
+			if (wjob->needToWait())
+				bt::WaitJob::execute(wjob);
+			else
+				delete wjob;
+		}
+		catch (Error & err)
+		{
+			Out(SYS_GEN|LOG_NOTICE) << "Error when unloading plugin: " << err.toString() << endl;
+		}
+		
 
 		gui->removePluginGui(p);
 		p->unload();
@@ -146,6 +165,29 @@ namespace kt
 
 	void PluginManager::unloadAll(bool save)
 	{
+		// first properly shutdown all plugins
+		bt::WaitJob* wjob = new WaitJob(2000);
+		try
+		{
+			bt::PtrMap<QString,Plugin>::iterator i = plugins.begin();
+			while (i != plugins.end())
+			{
+				Plugin* p = i->second;
+				p->shutdown(wjob);
+				i++;
+			}
+			
+			if (wjob->needToWait())
+				bt::WaitJob::execute(wjob);
+			else
+				delete wjob;
+		}
+		catch (Error & err)
+		{
+			Out(SYS_GEN|LOG_NOTICE) << "Error when unloading all plugins: " << err.toString() << endl;
+		}
+		
+		// then unload them
 		bt::PtrMap<QString,Plugin>::iterator i = plugins.begin();
 		while (i != plugins.end())
 		{
@@ -260,10 +302,11 @@ namespace kt
 		}
 
 		QTextStream out(&f);
-		out << "infowidgetplugin" << endl << "searchplugin" << endl;
+		
+		out << "Info Widget" << endl << "Search" << endl;
 
 		pltoload.clear();
-		pltoload.append("infowidgetplugin");
-		pltoload.append("searchplugin");
+		pltoload.append("Info Widget");
+		pltoload.append("Search");
 	}
 }
