@@ -41,6 +41,7 @@
 #include <util/error.h>
 #include "cachefile.h"
 #include "preallocationthread.h"
+#include "settings.h"
 
 
 // Not all systems have an O_LARGEFILE - Solaris depending
@@ -59,7 +60,9 @@ namespace bt
 {
 
 	CacheFile::CacheFile() : fd(-1),max_size(0),file_size(0),mutex(true)
-	{}
+	{
+		read_only = false;
+	}
 
 
 	CacheFile::~CacheFile()
@@ -73,15 +76,25 @@ namespace bt
 		path = npath;
 	}
 	
-	void CacheFile::openFile()
+	void CacheFile::openFile(Mode mode)
 	{
-		fd = ::open(QFile::encodeName(path),O_RDWR | O_LARGEFILE);
+		int flags = O_LARGEFILE;
+		
+		// by default allways try read write
+		fd = ::open(QFile::encodeName(path),flags | O_RDWR);
+		if (fd < 0 && mode == READ)
+		{
+			// in case RDWR fails, try readonly if possible
+			fd = ::open(QFile::encodeName(path),flags | O_RDONLY);
+			if (fd >= 0)
+				read_only = true;
+		}
 		
 		if (fd < 0)
 		{
 			throw Error(i18n("Cannot open %1 : %2").arg(path).arg(strerror(errno)));
 		}
-
+		
 		file_size = FileSize(fd);
 	}
 	
@@ -100,7 +113,12 @@ namespace bt
 		if (fd == -1)
 		{
 		//	Out() << "Reopening " << path << endl;
-			openFile();
+			openFile(mode);
+		}
+		
+		if (read_only && mode != READ)
+		{
+			throw Error(i18n("Cannot open %1 for writing : readonly filesystem").arg(path));
 		}
 		
 		if (off + size > max_size)
@@ -195,9 +213,12 @@ namespace bt
 		if (fd == -1)
 		{
 		//	Out() << "Reopening " << path << endl;
-			openFile();
+			openFile(RW);
 		}
 		
+		if (read_only)
+			throw Error(i18n("Cannot open %1 for writing : readonly filesystem").arg(path));
+		
 		// jump to the end of the file
 		SeekFile(fd,0,SEEK_END);
 		
@@ -213,16 +234,13 @@ namespace bt
 		// write data until to_write is 0
 		while (to_write > 0)
 		{
-			if (to_write < 1024)
-			{
-				::write(fd,buf,to_write);
-				to_write = 0;
-			}
-			else
-			{
-				::write(fd,buf,1024);
-				to_write -= 1024;
-			}
+			int nb = to_write > 1024 ? 1024 : to_write;
+			int ret = ::write(fd,buf,nb);
+			if (ret < 0)
+				throw Error(i18n("Cannot expand file %1 : %2").arg(path).arg(strerror(errno)));
+			else if (ret != nb)
+				throw Error(i18n("Cannot expand file %1 : incomplete write").arg(path));
+			to_write -= nb;
 		}
 		file_size += num;
 //		
@@ -324,7 +342,7 @@ namespace bt
 		if (fd == -1)
 		{
 		//	Out() << "Reopening " << path << endl;
-			openFile();
+			openFile(READ);
 			close_again = true;
 		}
 		
@@ -356,10 +374,13 @@ namespace bt
 		if (fd == -1)
 		{
 		//	Out() << "Reopening " << path << endl;
-			openFile();
+			openFile(RW);
 			close_again = true;
 		}
 		
+		if (read_only)
+			throw Error(i18n("Cannot open %1 for writing : readonly filesystem").arg(path));
+		
 		if (off + size > max_size)
 		{
 			Out() << "Warning : writing past the end of " << path << endl;
@@ -405,17 +426,43 @@ namespace bt
 	{
 		QMutexLocker lock(&mutex);
 		
+		if (FileSize(path) == max_size)
+		{
+			Out(SYS_GEN|LOG_NOTICE) << "File " << path << " already big enough" << endl;
+			return;
+		}
+		
 		Out(SYS_GEN|LOG_NOTICE) << "Preallocating file " << path << " (" << max_size << " bytes)" << endl;
 		bool close_again = false;
 		if (fd == -1)
 		{
-			openFile();
+			openFile(RW);
 			close_again = true;
 		}
+		
+		if (read_only)
+		{
+			if (close_again)
+				closeTemporary();
+			
+			throw Error(i18n("Cannot open %1 for writing : readonly filesystem").arg(path));
+		}
 
 		try
 		{
-			bt::TruncateFile(fd,max_size);
+			bool res = false;
+			
+			#ifdef HAVE_XFS_XFS_H
+				if( (! res) && Settings::fullDiskPrealloc() && (Settings::fullDiskPreallocMethod() == 1) )
+				{
+					res = XfsPreallocate(fd, max_size);
+				}
+			#endif
+			
+			if(! res)
+			{
+				bt::TruncateFile(fd,max_size,!Settings::fullDiskPrealloc());
+			}
 		}
 		catch (bt::Error & e)
 		{
@@ -435,5 +482,26 @@ namespace bt
 			closeTemporary();
 	}
 
-	
+	Uint64 CacheFile::diskUsage()
+	{
+		Uint64 ret = 0;
+		bool close_again = false;
+		if (fd == -1)
+		{
+			openFile(READ);
+			close_again = true;
+		}
+
+		struct stat sb;
+		if (fstat(fd,&sb) == 0)
+		{
+			ret = (Uint64)sb.st_blocks * 512;
+		}
+		
+	//	Out(SYS_GEN|LOG_NOTICE) << "CF: " << path << " is taking up " << ret << " bytes" << endl;
+		if (close_again)
+			closeTemporary();
+
+		return ret;
+	}
 }
