@@ -117,14 +117,18 @@ namespace bt
 		 * Preallocate disk space
 		 */
 		void preallocate(PreallocationThread* prealloc);
+
+		/// Get the number of bytes this cache file is taking up
+		Uint64 diskUsage();
 		
 	private:
 		void growFile(Uint64 to_write);
 		void closeTemporary();
-		void openFile();
+		void openFile(Mode mode);
 		
 	private:
 		int fd;
+		bool read_only;
 		Uint64 max_size,file_size;
 		QString path;
 		struct Entry
