@@ -30,6 +30,11 @@
 
 class QStringList;
 
+namespace KIO
+{
+	class Job;
+}
+
 namespace bt
 {
 	class Torrent;
@@ -87,6 +92,8 @@ namespace bt
 		/// Get the actual output path
 		QString getOutputPath() const;
 		
+		void changeOutputPath(const QString& output_path);
+		
 		/// Remove obsolete chunks
 		void checkMemoryUsage();
 		
@@ -96,6 +103,19 @@ namespace bt
 		 */
 		void changeDataDir(const QString & data_dir);
 		
+		/**
+		 * Move the data files of the torrent.
+		 * @param ndir The new directory
+		 * @return The job doing the move
+		 */
+		KIO::Job* moveDataFiles(const QString & ndir);
+		
+		/**
+		 * The move data files job has finished
+		 * @param job The move job
+		 */
+		void moveDataFilesCompleted(KIO::Job* job);
+		
 		/**
 		 * Loads the index file.
 		 * @throw Error When it can be loaded
@@ -104,9 +124,10 @@ namespace bt
 		
 		/**
 		 * Create the cache file, and index files.
+		 * @param check_priority Make sure chunk priorities and dnd status of files match
 		 * @throw Error When it can be created
 		 */
-		void createFiles();
+		void createFiles(bool check_priority = false);
 		
 		/**
 		 * Test all files and see if they are not missing.
@@ -270,7 +291,7 @@ namespace bt
 		 */
 		void include(Uint32 from,Uint32 to);
 		
-		
+				
 		/**
 		 * Data has been checked, and these chunks are OK.
 		 * The ChunkManager will update it's internal structures
@@ -295,6 +316,9 @@ namespace bt
 		
 		/// Set the maximum chunk size for a data check, 0 means alllways check
 		static void setMaxChunkSizeForDataCheck(Uint32 mcs) {max_chunk_size_for_data_check = mcs;}
+
+		/// Get the current disk usage of all the files in this torrent
+		Uint64 diskUsage();
 	signals:
 		/**
 		 * Emitted when a range of chunks has been excluded
