@@ -48,7 +48,7 @@ namespace bt
 	HTTPTracker::HTTPTracker(const KURL & url,kt::TorrentInterface* tor,const PeerID & id,int tier)
 		: Tracker(url,tor,id,tier)
 	{
-		active_job = active_scrape_job = 0;
+		active_job = 0;
 		
 		interval = 5 * 60; // default interval 5 minutes
 		failures = 0;
@@ -117,18 +117,7 @@ namespace bt
 	
 		Out(SYS_TRK|LOG_NOTICE) << "Doing scrape request to url : " << scrape_url.prettyURL() << endl;
 		KIO::MetaData md;
-		md["UserAgent"] = "ktorrent/" VERSION;
-		md["SendLanguageSettings"] = "false";
-		md["Cookies"] = "none";
-	//	md["accept"] = "text/plain";
-		md["accept"] = "text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2";
-		if (Settings::doNotUseKDEProxy())
-		{
-			// set the proxy if the doNotUseKDEProxy ix enabled (URL must be valid to)
-			KURL url = KURL::fromPathOrURL(Settings::httpTrackerProxy());
-			if (url.isValid())
-				md["UseProxy"] = url.pathOrURL();
-		}
+		setupMetaData(md);
 		
 		KIO::StoredTransferJob* j = KIO::storedGet(scrape_url,false,false);
 		// set the meta data
@@ -136,7 +125,6 @@ namespace bt
 		KIO::Scheduler::scheduleJob(j);
 		
 		connect(j,SIGNAL(result(KIO::Job* )),this,SLOT(onScrapeResult( KIO::Job* )));
-		active_scrape_job = j;
 	}
 	
 	void HTTPTracker::onScrapeResult(KIO::Job* j)
@@ -235,35 +223,19 @@ namespace bt
 
 
 		u.setEncodedPathAndQuery(epq);
-	
-		Out(SYS_TRK|LOG_NOTICE) << "Doing tracker request to url : " << u.prettyURL() << endl;
-
-		KIO::MetaData md;
-		md["UserAgent"] = "ktorrent/" VERSION;
-		md["SendLanguageSettings"] = "false";
-		md["Cookies"] = "none";
-	//	md["accept"] = "text/plain";
-		md["accept"] = "text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2";
-		if (Settings::doNotUseKDEProxy())
+		
+		if (active_job)
 		{
-			// set the proxy if the doNotUseKDEProxy ix enabled (URL must be valid to)
-			KURL url = KURL::fromPathOrURL(Settings::httpTrackerProxy());
-			if (url.isValid())
-				md["UseProxy"] = url.pathOrURL();
+			announce_queue.append(u);
+			Out(SYS_TRK|LOG_NOTICE) << "Announce ongoing, queueing announce" << endl;
+		}
+		else
+		{
+			doAnnounce(u);
+			// if there is a wait job, add this job to the waitjob 
+			if (wjob)
+				wjob->addExitOperation(new kt::ExitJobOperation(active_job));
 		}
-		
-		KIO::StoredTransferJob* j = KIO::storedGet(u,false,false);
-		// set the meta data
-		j->setMetaData(md);
-		KIO::Scheduler::scheduleJob(j);
-		
-		connect(j,SIGNAL(result(KIO::Job* )),this,SLOT(onAnnounceResult( KIO::Job* )));
-		
-		active_job = j;
-		// if there is a wait job, add this job to the waitjob 
-		if (wjob)
-			wjob->addExitOperation(new kt::ExitJobOperation(j));
-		requestPending();
 	}
 
 	bool HTTPTracker::updateData(const QByteArray & data)
@@ -385,17 +357,14 @@ namespace bt
 	
 	void HTTPTracker::onAnnounceResult(KIO::Job* j)
 	{
-		if (j != active_job)
-		{
-			return;
-		}
-		
 		if (j->error())
 		{
-			Out(SYS_TRK|LOG_IMPORTANT) << "Error : " << j->errorString() << endl;
+			KIO::StoredTransferJob* st = (KIO::StoredTransferJob*)j;
+			KURL u = st->url();
 			active_job = 0;
 			
-			if (event != "stopped")
+			Out(SYS_TRK|LOG_IMPORTANT) << "Error : " << st->errorString() << endl;
+			if (u.queryItem("event") != "stopped")
 			{
 				failures++;
 				requestFailed(j->errorString());
@@ -407,9 +376,11 @@ namespace bt
 		}
 		else
 		{
-			KIO::StoredTransferJob* st = (KIO::StoredTransferJob*)active_job;
+			KIO::StoredTransferJob* st = (KIO::StoredTransferJob*)j;
+			KURL u = st->url();
 			active_job = 0;
-			if (event != "stopped")
+			
+			if (u.queryItem("event") != "stopped")
 			{
 				try
 				{
@@ -418,7 +389,7 @@ namespace bt
 						failures = 0;
 						peersReady(this);
 						requestOK();
-						if (event == "started")
+						if (u.queryItem("event") == "started")
 							started = true;
 					}
 				}
@@ -435,6 +406,7 @@ namespace bt
 				stopDone();
 			}
 		}
+		doAnnounceQueue();
 	}
 
 	void HTTPTracker::emitInvalidURLFailure()
@@ -442,5 +414,49 @@ namespace bt
 		failures++;
 		requestFailed(i18n("Invalid tracker URL"));
 	}
+	
+	void HTTPTracker::setupMetaData(KIO::MetaData & md)
+	{
+		md["UserAgent"] = "ktorrent/" VERSION;
+		md["SendLanguageSettings"] = "false";
+		md["Cookies"] = "none";
+	//	md["accept"] = "text/plain";
+		md["accept"] = "text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2";
+		if (Settings::doNotUseKDEProxy())
+		{
+			// set the proxy if the doNotUseKDEProxy ix enabled (URL must be valid to)
+			KURL url = KURL::fromPathOrURL(Settings::httpTrackerProxy());
+			if (url.isValid())
+				md["UseProxy"] = url.pathOrURL();
+			else
+				md["UseProxy"] = QString::null;
+		}
+	}
+	
+	void HTTPTracker::doAnnounceQueue()
+	{
+		if (announce_queue.empty())
+			return;
+		
+		KURL u = announce_queue.front();
+		announce_queue.pop_front();
+		doAnnounce(u);
+	}
+	
+	void HTTPTracker::doAnnounce(const KURL & u)
+	{
+		Out(SYS_TRK|LOG_NOTICE) << "Doing tracker request to url : " << u.prettyURL() << endl;
+		KIO::MetaData md;
+		setupMetaData(md);
+		KIO::StoredTransferJob* j = KIO::storedGet(u,false,false);
+		// set the meta data
+		j->setMetaData(md);
+		KIO::Scheduler::scheduleJob(j);
+		
+		connect(j,SIGNAL(result(KIO::Job* )),this,SLOT(onAnnounceResult( KIO::Job* )));
+		
+		active_job = j;
+		requestPending();
+	}
 }
 #include "httptracker.moc"
