@@ -22,6 +22,7 @@
 #include <qstringlist.h>
 #include <qfileinfo.h>
 #include <klocale.h>
+#include <kio/netaccess.h>
 #include <util/file.h>
 #include <util/fileops.h>
 #include <util/functions.h>
@@ -35,6 +36,7 @@
 #include "cachefile.h"
 #include "dndfile.h"
 #include "preallocationthread.h"
+#include "movedatafilesjob.h"
 
 
 
@@ -42,6 +44,7 @@ namespace bt
 {
 	static Uint64 FileOffset(Chunk* c,const TorrentFile & f,Uint64 chunk_size);
 	static Uint64 FileOffset(Uint32 cindex,const TorrentFile & f,Uint64 chunk_size);
+	static void DeleteEmptyDirs(const QString & output_dir,const QString & fpath);
 
 
 	MultiFileCache::MultiFileCache(Torrent& tor,const QString & tmpdir,const QString & datadir,bool custom_output_name) : Cache(tor, tmpdir,datadir)
@@ -164,6 +167,90 @@ namespace bt
 			}
 		}
 	}
+	
+	void MultiFileCache::changeOutputPath(const QString & outputpath)
+	{
+		output_dir = outputpath;
+		if (!output_dir.endsWith(bt::DirSeparator()))
+			output_dir += bt::DirSeparator();
+		
+		datadir = output_dir;
+		
+		if (!bt::Exists(cache_dir))
+			MakeDir(cache_dir);
+		
+		for (Uint32 i = 0;i < tor.getNumFiles();i++)
+		{
+			TorrentFile & tf = tor.getFile(i);
+			if (!tf.doNotDownload())
+			{
+				QString fpath = tf.getPath();
+				if (bt::Exists(output_dir + fpath))
+				{
+					bt::Delete(cache_dir + fpath,true); // delete any existing symlinks
+					// create new one
+					bt::SymLink(output_dir + fpath,cache_dir + fpath,true); 
+				}
+			}
+		}
+	}
+	
+	KIO::Job* MultiFileCache::moveDataFiles(const QString & ndir)
+	{
+		if (!bt::Exists(ndir))
+			bt::MakeDir(ndir);
+		
+		QString nd = ndir;
+		if (!nd.endsWith(bt::DirSeparator()))
+			nd += bt::DirSeparator();
+		
+		try
+		{
+			MoveDataFilesJob* mvd = new MoveDataFilesJob();
+			for (Uint32 i = 0;i < tor.getNumFiles();i++)
+			{			
+				TorrentFile & tf = tor.getFile(i);
+				if (tf.doNotDownload())
+					continue;
+				
+				// check if every directory along the path exists, and if it doesn't
+				// create it
+				QStringList sl = QStringList::split(bt::DirSeparator(),nd + tf.getPath());
+				QString odir = bt::DirSeparator();
+				for (Uint32 i = 0;i < sl.count() - 1;i++)
+				{
+					odir += sl[i] + bt::DirSeparator();
+					if (!bt::Exists(odir))
+					{
+						bt::MakeDir(odir);
+					}
+				}
+				
+				mvd->addMove(output_dir + tf.getPath(),nd + tf.getPath());
+			}
+			
+			mvd->startMoving();
+			return mvd;
+		}
+		catch (bt::Error & err)
+		{
+			throw; // rethrow error
+		}
+		return 0;
+	}
+	
+	void MultiFileCache::moveDataFilesCompleted(KIO::Job* job)
+	{
+		if (!job->error())
+		{
+			for (Uint32 i = 0;i < tor.getNumFiles();i++)
+			{
+				TorrentFile & tf = tor.getFile(i);
+					// check for empty directories and delete them 
+				DeleteEmptyDirs(output_dir,tf.getPath());
+			}
+		}
+	}
 
 	void MultiFileCache::create()
 	{
@@ -174,8 +261,9 @@ namespace bt
 		if (!bt::Exists(tmpdir + "dnd"))
 			bt::MakeDir(tmpdir + "dnd");
 
+		// update symlinks
 		for (Uint32 i = 0;i < tor.getNumFiles();i++)
-		{
+		{			
 			TorrentFile & tf = tor.getFile(i);
 			touch(tf);
 		}
@@ -248,11 +336,20 @@ namespace bt
 			if (!fd)
 				return;
 			
-			Uint64 off = FileOffset(c,f,tor.getChunkSize());
-			Uint8* buf = (Uint8*)fd->map(c,off,c->getSize(),CacheFile::READ);
-			if (buf)
-				c->setData(buf,Chunk::MMAPPED);
-			return;
+			if (Cache::mappedModeAllowed() && mmap_failures < 3)
+			{
+				Uint64 off = FileOffset(c,f,tor.getChunkSize());
+				Uint8* buf = (Uint8*)fd->map(c,off,c->getSize(),CacheFile::READ);
+				if (buf)
+				{
+					c->setData(buf,Chunk::MMAPPED);
+					// only return when the mapping is OK
+					// if mmap fails we will just load it buffered
+					return;
+				}
+				else
+					mmap_failures++;
+			}
 		}
 		
 		Uint8* data = new Uint8[c->getSize()];
@@ -317,12 +414,16 @@ namespace bt
 			Uint64 off = FileOffset(c,tor.getFile(tflist.first()),tor.getChunkSize());
 			CacheFile* fd = files.find(tflist.first());
 			Uint8* buf = 0;
-			if (fd)
+			if (fd && Cache::mappedModeAllowed() && mmap_failures < 3)
+			{
 				buf = (Uint8*)fd->map(c,off,c->getSize(),CacheFile::RW);
+				if (!buf)
+					mmap_failures++;
+			}
+			
 			if (!buf)
 			{
-				// if mmap fails use buffered mode
-				Out() << "Warning : mmap failed, falling back to buffered mode" << endl;
+				// if mmap fails or is not possible use buffered mode
 				c->allocate();
 				c->setStatus(Chunk::BUFFERED);
 			}
@@ -534,7 +635,19 @@ namespace bt
 		// truncate it
 		try
 		{
-			bt::TruncateFile(output_file,tf->getSize());
+			bool res = false;
+			
+			#ifdef HAVE_XFS_XFS_H
+				if( (! res) && (Settings::fullDiskPreallocMethod() == 1) )
+				{
+					res = XfsPreallocate(output_file, tf->getSize());
+				}
+			#endif
+			
+			if(! res)
+			{
+				bt::TruncateFile(output_file,tf->getSize());
+			}
 		}
 		catch (bt::Error & e)
 		{
@@ -663,7 +776,7 @@ namespace bt
 			{
 				// no childern so delete the directory
 				Out(SYS_GEN|LOG_IMPORTANT) << "Deleting empty directory : " << path << endl;
-				bt::Delete(path);
+				bt::Delete(path,true);
 				sl.pop_back(); // remove the last so we can go one higher
 			}
 			else
@@ -682,7 +795,7 @@ namespace bt
 		if (el.count() == 0)
 		{
 			Out(SYS_GEN|LOG_IMPORTANT) << "Deleting empty directory : " << output_dir << endl;
-			bt::Delete(output_dir);
+			bt::Delete(output_dir,true);
 		}
 	}
 	
@@ -691,17 +804,53 @@ namespace bt
 		for (Uint32 i = 0;i < tor.getNumFiles();i++)
 		{
 			TorrentFile & tf = tor.getFile(i);
-			if (tf.doNotDownload())
-				continue;
-			
 			QString fpath = tf.getPath();
-			// first delete the file
-			bt::Delete(output_dir + fpath);
+			if (!tf.doNotDownload())
+			{
+				// first delete the file
+				bt::Delete(output_dir + fpath);
+			}
 			
 			// check for subdirectories
 			DeleteEmptyDirs(output_dir,fpath);
 		}
 	}
+	
+	Uint64 MultiFileCache::diskUsage()
+	{
+		Uint64 sum = 0;
+
+		for (Uint32 i = 0;i < tor.getNumFiles();i++)
+		{
+			TorrentFile & tf = tor.getFile(i);
+			if (tf.doNotDownload())
+				continue;
+
+			try
+			{
+				CacheFile* cf = files.find(i);
+				if (cf)
+				{
+					sum += cf->diskUsage();
+				}
+				else
+				{
+					// doesn't exist yet, must be before open is called
+					// so create one and delete it right after
+					cf = new CacheFile();
+					cf->open(cache_dir + tf.getPath(),tf.getSize());
+					sum += cf->diskUsage();
+					delete cf;
+				}
+			}
+			catch (bt::Error & err) // make sure we catch any exceptions
+			{
+				Out(SYS_DIO|LOG_DEBUG) << "Error: " << err.toString() << endl;
+			}
+		}
+
+		return sum;
+	}
 
 	///////////////////////////////
 
@@ -716,4 +865,3 @@ namespace bt
 	}
 	
 }
-
