@@ -262,6 +262,10 @@ namespace bt
 	
 	void PeerDownloader::choked()
 	{
+		// choke doesn't mean reject when fast extensions are enabled
+		if (peer->getStats().fast_extensions)
+			return;
+		
 		QValueList<TimeStampedRequest>::iterator i = reqs.begin();
 		while (i != reqs.end())
 		{
@@ -286,7 +290,7 @@ namespace bt
 		// modify the interval if necessary
 		double pieces_per_sec = (double)peer->getDownloadRate() / MAX_PIECE_LEN;
 		
-		Uint32 max_reqs = 1 + (Uint32)ceil(8*pieces_per_sec);
+		Uint32 max_reqs = 1 + (Uint32)ceil(10*pieces_per_sec);
 		
 		while (wait_queue.count() > 0 && reqs.count() < max_reqs)
 		{
