@@ -126,6 +126,8 @@ namespace bt
 		static void setMaxTotalConnections(Uint32 max);
 		static Uint32 getMaxTotalConnections() {return max_total_connections;}
 		
+		static Uint32 getTotalConnections() {return total_connections;}
+		
 		/// Is the peer manager started
 		bool isStarted() const {return started;}
 
@@ -196,6 +198,9 @@ namespace bt
 		/// Enable or disable PEX
 		void setPexEnabled(bool on);
 		
+		/// Set the group IDs of each peer
+		void setGroupIDs(Uint32 up,Uint32 down);
+		
 	public slots:
 		/**
 		 * A PeerSource, has new potential peers.
