@@ -52,9 +52,13 @@ namespace bt
 		cache_file = tmpdir + "cache";
 	}
 	
-	void SingleFileCache::moveDataFiles(const QString & ndir)
+	KIO::Job* SingleFileCache::moveDataFiles(const QString & ndir)
+	{
+		return KIO::move(KURL::fromPathOrURL(output_file),KURL::fromPathOrURL(ndir));
+	}
+	
+	void SingleFileCache::moveDataFilesCompleted(KIO::Job* /*job*/)
 	{
-		bt::Move(output_file,ndir);
 	}
 	
 	void bt::SingleFileCache::changeOutputPath(const QString & outputpath)
