@@ -65,9 +65,10 @@
 #include "announcelist.h"
 #include "preallocationthread.h"
 #include "timeestimator.h"
+#include "settings.h"
 
-#include <util/profiler.h>
-
+#include <net/socketmonitor.h>
+		
 
 using namespace kt;
 
@@ -102,8 +103,10 @@ namespace bt
 		istats.trk_prev_bytes_dl = istats.trk_prev_bytes_ul = 0;
 		istats.io_error = false;
 		istats.priority = 0;
-		istats.maxShareRatio = 0.00f;
+		stats.max_share_ratio = 0.00f;
 		istats.custom_output_name = false;
+		istats.diskspace_warning_emitted = false;
+		stats.max_seed_time = 0;
 		updateStats();
 		prealoc_thread = 0;
 		dcheck_thread = 0;
@@ -111,6 +114,10 @@ namespace bt
 		stats.num_corrupted_chunks = 0;
 		
 		m_eta = new TimeEstimator(this);
+		// by default no torrent limits
+		upload_gid = download_gid = 0;
+		upload_limit = download_limit = 0;
+		moving_files = false;
 	}
 
 
@@ -136,7 +143,7 @@ namespace bt
 	void TorrentControl::update()
 	{
 		UpdateCurrentTime();
-		if (stats.status == kt::CHECKING_DATA)
+		if (stats.status == kt::CHECKING_DATA || moving_files)
 			return;
 		
 		if (istats.io_error)
@@ -181,6 +188,9 @@ namespace bt
 			// first update peermanager
 			pman->update();
 			bool comp = stats.completed;
+			
+			//helper var, check if needed to move completed files somewhere
+			bool moveCompleted = false;
 
 			// then the downloader and uploader
 			up->update(choke->getOptimisticlyUnchokedPeerID());			
@@ -190,7 +200,6 @@ namespace bt
 			if (stats.completed && !comp)
 			{
 				pman->killSeeders();
-				pman->killUninterested();
 				QDateTime now = QDateTime::currentDateTime();
 				istats.running_time_dl += istats.time_started_dl.secsTo(now);
 				updateStatusMsg();
@@ -202,6 +211,12 @@ namespace bt
 					psman->completed();
 				
 				finished(this);
+				
+				//Move completed download to specified directory if needed
+				if(Settings::useCompletedDir())
+				{
+					moveCompleted = true;
+				}
 			}
 			else if (!stats.completed && comp)
 			{
@@ -229,7 +244,6 @@ namespace bt
 				if (stats.completed)
 				{
 					pman->killSeeders();
-					pman->killUninterested();
 				}
 				
 				doChoking();
@@ -261,7 +275,7 @@ namespace bt
 				stalled_timer.update();
 			}
 			
-			if(overMaxRatio()) 
+			if(overMaxRatio() || overMaxSeedTime()) 
 			{ 
 				if(istats.priority!=0) //if it's queued make sure to dequeue it 
 				{
@@ -270,9 +284,24 @@ namespace bt
 				}
                  
 				stop(true); 
-				emit seedingAutoStopped(this);
-            } 
-
+				emit seedingAutoStopped(this, overMaxRatio() ? kt::MAX_RATIO_REACHED : kt::MAX_SEED_TIME_REACHED);
+            }
+			
+			//Update diskspace if needed (every 1 min)			
+			if(!stats.completed && stats.running && bt::GetCurrentTime() - last_diskspace_check >= 60 * 1000)
+			{
+				checkDiskSpace(true);
+			}
+			
+			//Move completed files if needed:
+			if (moveCompleted)
+			{
+				QString outdir = Settings::completedDir();
+				if(!outdir.endsWith(bt::DirSeparator()))
+					outdir += bt::DirSeparator();
+				
+				changeOutputDir(outdir);
+			}
 		}
 		catch (Error & e)
 		{
@@ -292,10 +321,11 @@ namespace bt
 	void TorrentControl::start()
 	{	
 		// do not start running torrents
-		if (stats.running || stats.status == kt::ALLOCATING_DISKSPACE)
+		if (stats.running || stats.status == kt::ALLOCATING_DISKSPACE || moving_files)
 			return;
 
 		stats.stopped_by_error = false;
+		istats.diskspace_warning_emitted = false;
 		istats.io_error = false;
 		try
 		{	
@@ -326,12 +356,20 @@ namespace bt
 		
 		if (prealloc)
 		{
-			Out(SYS_GEN|LOG_NOTICE) << "Pre-allocating diskspace" << endl;
-			prealoc_thread = new PreallocationThread(cman);
-			stats.running = true;
-			stats.status = kt::ALLOCATING_DISKSPACE;
-			prealoc_thread->start();
-			return;
+			// only start preallocation if we are allowed by the settings
+			if (Settings::diskPrealloc() && !cman->haveAllChunks())
+			{
+				Out(SYS_GEN|LOG_NOTICE) << "Pre-allocating diskspace" << endl;
+				prealoc_thread = new PreallocationThread(cman);
+				stats.running = true;
+				stats.status = kt::ALLOCATING_DISKSPACE;
+				prealoc_thread->start();
+				return;
+			}
+			else
+			{
+				prealloc = false;
+			}
 		}
 		
 		continueStart();
@@ -435,6 +473,8 @@ namespace bt
 		updateStats();
 		stats.trk_bytes_downloaded = 0;
 		stats.trk_bytes_uploaded = 0;
+		
+		emit torrentStopped(this);
 	}
 
 	void TorrentControl::setMonitor(kt::MonitorInterface* tmo)
@@ -466,7 +506,7 @@ namespace bt
 			delete tor;
 			tor = 0;
 			throw Error(i18n("An error occurred while loading the torrent."
-					" The torrent is probably corrupt or is not a torrent file."));
+					" The torrent is probably corrupt or is not a torrent file.\n%1").arg(torrent));
 		}
 		
 		initInternal(qman,tmpdir,ddir,default_save_dir,torrent.startsWith(tmpdir));
@@ -659,11 +699,11 @@ namespace bt
 		updateStats();
 		saveStats();
 		stats.output_path = cman->getOutputPath();
-		if (stats.output_path.isNull())
+	/*	if (stats.output_path.isNull())
 		{
 			cman->createFiles();
 			stats.output_path = cman->getOutputPath();
-		}
+		}*/
 		Out() << "OutputPath = " << stats.output_path << endl;
 	}
 	
@@ -723,6 +763,9 @@ namespace bt
 				p->emitPortPacket();
 		}
 		
+		// set group ID's for traffic shaping
+		p->setGroupIDs(upload_gid,download_gid);
+		
 		if (tmon)
 			tmon->peerAdded(p);
 	}
@@ -767,6 +810,105 @@ namespace bt
 		cman->changeDataDir(datadir);
 		return true;
 	}
+	
+	bool TorrentControl::changeOutputDir(const QString & new_dir, bool moveFiles)
+	{
+		if (moving_files)
+			return false;
+		
+		Out(SYS_GEN|LOG_NOTICE) << "Moving data for torrent " << stats.torrent_name << " to " << new_dir << endl;
+		
+		restart_torrent_after_move_data_files = false;
+		
+		//check if torrent is running and stop it before moving data
+		if(stats.running)
+		{
+			restart_torrent_after_move_data_files = true;
+			this->stop(false);
+		}
+		
+		moving_files = true;
+		try
+		{
+			QString nd;
+			if (istats.custom_output_name)
+			{
+				int slash_pos = stats.output_path.findRev(bt::DirSeparator(),-2);
+				nd = new_dir + stats.output_path.mid(slash_pos + 1);
+			}
+			else
+			{
+				nd = new_dir + tor->getNameSuggestion();
+			}
+			
+			if (stats.output_path != nd)
+			{
+				KIO::Job* j = 0;
+				if(moveFiles)
+				{	
+					if (stats.multi_file_torrent)
+						j = cman->moveDataFiles(nd);
+					else
+						j = cman->moveDataFiles(new_dir);
+				}
+				
+				move_data_files_destination_path = nd;
+				if (j)
+				{
+					connect(j,SIGNAL(result(KIO::Job*)),this,SLOT(moveDataFilesJobDone(KIO::Job*)));
+					return true;
+				}
+				else
+				{
+					moveDataFilesJobDone(0);
+				}
+			}
+			else
+			{
+				Out(SYS_GEN|LOG_NOTICE) << "Source is the same as destination, so doing nothing" << endl;
+			}
+		}
+		catch (Error& err)
+		{			
+			Out(SYS_GEN|LOG_IMPORTANT) << "Could not move " << stats.output_path << " to " << new_dir << ". Exception: " << err.toString() << endl;
+			moving_files = false;
+			return false;
+		}
+		
+		moving_files = false;
+		if (restart_torrent_after_move_data_files)
+		{
+			this->start();
+		}
+		
+		return true;
+	}
+	
+	void TorrentControl::moveDataFilesJobDone(KIO::Job* job)
+	{
+		if (job)
+			cman->moveDataFilesCompleted(job);
+		
+		if (!job || (job && !job->error()))
+		{
+			cman->changeOutputPath(move_data_files_destination_path);
+			outputdir = stats.output_path = move_data_files_destination_path;
+			istats.custom_output_name = true;
+				
+			saveStats();
+			Out(SYS_GEN|LOG_NOTICE) << "Data directory changed for torrent " << "'" << stats.torrent_name << "' to: " << move_data_files_destination_path << endl;
+		}
+		else if (job->error())
+		{
+			Out(SYS_GEN|LOG_IMPORTANT) << "Could not move " << stats.output_path << " to " << move_data_files_destination_path << endl;
+		}
+		
+		moving_files = false;
+		if (restart_torrent_after_move_data_files)
+		{
+			this->start();
+		}
+	}
 
 
 	void TorrentControl::rollback()
@@ -791,7 +933,7 @@ namespace bt
 			stats.status = kt::NOT_STARTED;
 		else if(!stats.running && !stats.user_controlled)
 			stats.status = kt::QUEUED;
-		else if (!stats.running && stats.completed && overMaxRatio())
+		else if (!stats.running && stats.completed && (overMaxRatio() || overMaxSeedTime()))
 			stats.status = kt::SEEDING_COMPLETE;
 		else if (!stats.running && stats.completed)
 			stats.status = kt::DOWNLOAD_COMPLETE;
@@ -841,7 +983,7 @@ namespace bt
 	{
 		StatsFile st(datadir + "stats");
 
-		st.write("OUTPUTDIR", cman->getDataDir());
+		st.write("OUTPUTDIR", cman->getDataDir());			
 		
 		if (cman->getDataDir() != outputdir)
 			outputdir = cman->getDataDir();
@@ -864,7 +1006,8 @@ namespace bt
 		st.write("AUTOSTART", QString("%1").arg(stats.autostart));
 		st.write("IMPORTED", QString("%1").arg(stats.imported_bytes));
 		st.write("CUSTOM_OUTPUT_NAME",istats.custom_output_name ? "1" : "0");
-		st.write("MAX_RATIO", QString("%1").arg(istats.maxShareRatio,0,'f',2));
+		st.write("MAX_RATIO", QString("%1").arg(stats.max_share_ratio,0,'f',2));
+		st.write("MAX_SEED_TIME",QString::number(stats.max_seed_time));
 		st.write("RESTART_DISK_PREALLOCATION",prealloc ? "1" : "0");
 		
 		if(!stats.priv_torrent)
@@ -874,6 +1017,9 @@ namespace bt
 			st.write("UT_PEX", isFeatureEnabled(kt::UT_PEX_FEATURE) ? "1" : "0");
 		}
 		
+		st.write("UPLOAD_LIMIT",QString::number(upload_limit));
+		st.write("DOWNLOAD_LIMIT",QString::number(download_limit));
+		
 		st.writeSync();
 	}
 
@@ -902,10 +1048,12 @@ namespace bt
 		
 		stats.imported_bytes = st.readUint64("IMPORTED");
 		float rat = st.readFloat("MAX_RATIO");
-		istats.maxShareRatio = rat;
+		stats.max_share_ratio = rat;
 		if (st.hasKey("RESTART_DISK_PREALLOCATION"))
 			prealloc = st.readString("RESTART_DISK_PREALLOCATION") == "1";
 		
+		stats.max_seed_time = st.readFloat("MAX_SEED_TIME");
+		
 		if (!stats.priv_torrent)
 		{
 			if(st.hasKey("DHT"))
@@ -917,6 +1065,44 @@ namespace bt
 			if (st.hasKey("UT_PEX"))
 				setFeatureEnabled(kt::UT_PEX_FEATURE,st.readBoolean("UT_PEX"));
 		}
+		
+		net::SocketMonitor & smon = net::SocketMonitor::instance();
+		
+		Uint32 nl = st.readInt("UPLOAD_LIMIT");
+		if (nl != upload_limit)
+		{
+			if (nl > 0)
+			{
+				if (upload_gid)
+					smon.setGroupLimit(net::SocketMonitor::UPLOAD_GROUP,upload_gid,nl);
+				else
+					upload_gid = smon.newGroup(net::SocketMonitor::UPLOAD_GROUP,nl);
+			}
+			else
+			{
+				smon.removeGroup(net::SocketMonitor::UPLOAD_GROUP,upload_gid);
+				upload_gid = 0;
+			}
+		}
+		upload_limit = nl;
+		
+		nl = st.readInt("DOWNLOAD_LIMIT");
+		if (nl != download_limit)
+		{
+			if (nl > 0)
+			{
+				if (download_gid)
+					smon.setGroupLimit(net::SocketMonitor::DOWNLOAD_GROUP,download_gid,nl);
+				else
+					download_gid = smon.newGroup(net::SocketMonitor::DOWNLOAD_GROUP,nl);
+			}
+			else
+			{
+				smon.removeGroup(net::SocketMonitor::DOWNLOAD_GROUP,download_gid);
+				download_gid = 0;
+			}
+		}
+		download_limit = nl;
 	}
 
 	void TorrentControl::loadOutputDir()
@@ -966,10 +1152,8 @@ namespace bt
 		stats.num_chunks_downloaded = cman ? cman->chunksDownloaded() : 0;
 		stats.num_chunks_excluded = cman ? cman->chunksExcluded() : 0;
 		stats.chunk_size = tor ? tor->getChunkSize() : 0;
+		stats.num_chunks_left = cman ? cman->chunksLeft() : 0;
 		stats.total_bytes_to_download = (tor && cman) ?	tor->getFileLength() - cman->bytesExcluded() : 0;
-		stats.max_share_ratio = istats.maxShareRatio;
-		
-		
 		
 		if (stats.bytes_downloaded >= istats.prev_bytes_dl)
 			stats.session_bytes_downloaded = stats.bytes_downloaded - istats.prev_bytes_dl;
@@ -1128,24 +1312,43 @@ namespace bt
 	{
 		if(ratio == 1.00f)
 		{
-			if(istats.maxShareRatio != ratio)
-				istats.maxShareRatio = ratio;
+			if (stats.max_share_ratio != ratio)
+				stats.max_share_ratio = ratio;
 		}
 		else
-			istats.maxShareRatio = ratio;
+			stats.max_share_ratio = ratio;
 		
-		if(stats.completed && !stats.running && !stats.user_controlled && (kt::ShareRatio(stats) >= istats.maxShareRatio))
+		if(stats.completed && !stats.running && !stats.user_controlled && (kt::ShareRatio(stats) >= stats.max_share_ratio))
 			setPriority(0); //dequeue it
 		
 		saveStats();
 		emit maxRatioChanged(this);
 	}
 	
+	void TorrentControl::setMaxSeedTime(float hours)
+	{
+		stats.max_seed_time = hours;
+		saveStats();
+	}
+	
 	bool TorrentControl::overMaxRatio()
 	{
-		if(stats.completed && stats.bytes_uploaded != 0 && stats.bytes_downloaded != 0 && istats.maxShareRatio > 0)
+		if(stats.completed && stats.bytes_uploaded != 0 && stats.bytes_downloaded != 0 && stats.max_share_ratio > 0)
 		{
-			if(kt::ShareRatio(stats) >= istats.maxShareRatio)
+			if(kt::ShareRatio(stats) >= stats.max_share_ratio)
+				return true;
+		}
+		
+		return false;
+	}
+	
+	bool TorrentControl::overMaxSeedTime()
+	{
+		if(stats.completed && stats.bytes_uploaded != 0 && stats.bytes_downloaded != 0 && stats.max_seed_time > 0)
+		{
+			Uint32 dl = getRunningTimeDL();
+			Uint32 ul = getRunningTimeUL();
+			if ((ul - dl) / 3600.0f > stats.max_seed_time)
 				return true;
 		}
 		
@@ -1179,6 +1382,8 @@ namespace bt
 				return i18n("Queued");
 			case kt::CHECKING_DATA:
 				return i18n("Checking data");
+			case kt::NO_SPACE_LEFT:
+				return i18n("Stopped. No space left on device.");
 		}
 		return QString::null;
 	}
@@ -1447,9 +1652,119 @@ namespace bt
 	
 	void TorrentControl::createFiles()
 	{
-		cman->createFiles();
+		cman->createFiles(true);
 		stats.output_path = cman->getOutputPath();
 	}
+	
+	bool TorrentControl::checkDiskSpace(bool emit_sig)
+	{	
+		last_diskspace_check = bt::GetCurrentTime();
+		
+		//calculate free disk space
+		Uint64 bytes_free = 0;
+		if (FreeDiskSpace(getDataDir(),bytes_free))
+		{
+			Uint64 bytes_to_download = stats.total_bytes_to_download;
+			Uint64 downloaded = 0;
+			try
+			{
+				downloaded = cman->diskUsage();
+			}
+			catch (bt::Error & err)
+			{
+				Out(SYS_GEN|LOG_DEBUG) << "Error : " << err.toString() << endl;
+			}
+			Uint64 remaining = 0;
+			if (downloaded <= bytes_to_download)
+				remaining = bytes_to_download - downloaded;
+
+			if (remaining > bytes_free)
+			{
+				bool toStop = bytes_free < (Uint64) Settings::minDiskSpace() * 1024 * 1024;						
+				
+				// if we don't need to stop the torrent, only emit the signal once
+				// so that we do bother the user continously
+				if (emit_sig && (toStop || !istats.diskspace_warning_emitted))
+				{
+					emit diskSpaceLow(this, toStop);
+					istats.diskspace_warning_emitted = true; 
+				}
+				
+				if (!stats.running)
+				{
+					stats.status = NO_SPACE_LEFT;
+				}
+				
+				return false;
+			}
+		}
+		
+		return true;
+	}
+	
+	void TorrentControl::setTrafficLimits(Uint32 up,Uint32 down)
+	{
+		net::SocketMonitor & smon = net::SocketMonitor::instance();
+		if (up && !upload_gid)
+		{
+			// create upload group
+			upload_gid = smon.newGroup(net::SocketMonitor::UPLOAD_GROUP,up);
+			upload_limit = up;
+		}
+		else if (up && upload_gid)
+		{
+			// change existing group limit
+			smon.setGroupLimit(net::SocketMonitor::UPLOAD_GROUP,upload_gid,up);
+			upload_limit = up;
+		}
+		else if (!up && !upload_gid)
+		{
+			upload_limit = up;
+		}
+		else // !up && upload_gid
+		{
+			// remove existing group
+			smon.removeGroup(net::SocketMonitor::UPLOAD_GROUP,upload_gid);
+			upload_gid = upload_limit = 0;
+		}
+		
+		if (down && !download_gid)
+		{
+			// create download grodown
+			download_gid = smon.newGroup(net::SocketMonitor::DOWNLOAD_GROUP,down);
+			download_limit = down;
+		}
+		else if (down && download_gid)
+		{
+			// change existing grodown limit
+			smon.setGroupLimit(net::SocketMonitor::DOWNLOAD_GROUP,download_gid,down);
+			download_limit = down;
+		}
+		else if (!down && !download_gid)
+		{
+			download_limit = down;
+		}
+		else // !down && download_gid
+		{
+			// remove existing grodown
+			smon.removeGroup(net::SocketMonitor::DOWNLOAD_GROUP,download_gid);
+			download_gid = download_limit = 0;
+		}
+		
+		saveStats();
+		pman->setGroupIDs(upload_gid,download_gid);
+	}
+	
+	void TorrentControl::getTrafficLimits(Uint32 & up,Uint32 & down)
+	{
+		up = upload_limit;
+		down = download_limit;
+	}
+	
+	const PeerManager * TorrentControl::getPeerMgr() const
+	{
+		return pman;
+	}
 }
 
 #include "torrentcontrol.moc"
