@@ -22,6 +22,7 @@
 #endif
 
 #include <string.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 #include <klocale.h>
@@ -37,16 +38,41 @@
 #include "log.h"
 #include <torrent/globals.h>
 #include "file.h"
+#include "array.h"
+
+#ifdef HAVE_XFS_XFS_H
+
+#if !defined(HAVE___S64) || !defined(HAVE___U64)
+#include <stdint.h>
+#endif
+
+#ifndef HAVE___U64
+typedef	uint64_t	__u64;
+#endif
+
+#ifndef HAVE___S64
+typedef	int64_t		__s64;
+#endif
+
+#include <xfs/xfs.h>
+#endif
 
 #ifndef O_LARGEFILE
 #define O_LARGEFILE 0
 #endif
 
+#if HAVE_STATVFS
+#include <sys/statvfs.h>
+#else
+#include <sys/param.h>
+#include <sys/mount.h>
+#endif
+
 namespace bt
 {
 	void MakeDir(const QString & dir,bool nothrow)
 	{
-		if (mkdir(QFile::encodeName(dir),0755) < -1)
+		if (mkdir(QFile::encodeName(dir),0777) < -1)
 		{
 			if (!nothrow)
 				throw Error(i18n("Cannot create directory %1: %2")
@@ -150,20 +176,29 @@ namespace bt
 				continue;
 			
 			if (!DelDir(d.absFilePath(entry)))
+			{
+				Out(SYS_GEN|LOG_DEBUG) << "Delete of " << fn << "/" << entry << " failed !" << endl;
 				return false;	
+			}
 		}
 		
-		QStringList files = d.entryList(QDir::Files | QDir::System);
+		QStringList files = d.entryList(QDir::Files | QDir::System | QDir::Hidden);
 		for (QStringList::iterator i = files.begin(); i != files.end();i++)
 		{
 			QString entry = *i;
 
 			if (remove(QFile::encodeName(d.absFilePath(entry))) < 0)
+			{
+				Out(SYS_GEN|LOG_DEBUG) << "Delete of " << fn << "/" << entry << " failed !" << endl;
 				return false;	
+			}
 		}
 		
 		if (!d.rmdir(d.absPath()))
+		{
+			Out(SYS_GEN|LOG_DEBUG) << "Failed to remove " << d.absPath() << endl;
 			return false;
+		}
 		
 		return true;
 	}
@@ -255,17 +290,17 @@ namespace bt
 		
 		return (Uint64)sb.st_size;
 	}
-	
+
 	bool FatPreallocate(int fd,Uint64 size)
 	{
 		try
 		{
-			SeekFile(fd, size, SEEK_SET);
+			SeekFile(fd, size - 1, SEEK_SET);
 			char zero = 0;		
 			if (write(fd, &zero, 1) == -1)
 				return false;
 				
-			TruncateFile(fd,size);
+			TruncateFile(fd,size,true);
 		}
 		catch (bt::Error & e)
 		{
@@ -285,19 +320,82 @@ namespace bt
 		close(fd);
 		return ret;
 	}
-	
 
-	void TruncateFile(int fd,Uint64 size)
+#ifdef HAVE_XFS_XFS_H
+	
+	bool XfsPreallocate(int fd, Uint64 size)
+	{
+		if( ! platform_test_xfs_fd(fd) )
+		{
+			return false;
+		}
+		
+		xfs_flock64_t allocopt;
+		allocopt.l_whence = 0;
+		allocopt.l_start = 0;
+		allocopt.l_len  = size;
+		
+		return (! static_cast<bool>(xfsctl(0, fd, XFS_IOC_RESVSP64, &allocopt)) );
+		
+	}
+	
+	bool XfsPreallocate(const QString & path, Uint64 size)
+	{
+		int fd = ::open(QFile::encodeName(path), O_RDWR | O_LARGEFILE);
+		if (fd < 0)
+			throw Error(i18n("Cannot open %1 : %2").arg(path).arg(strerror(errno)));
+		
+		bool ret = XfsPreallocate(fd,size);
+		close(fd);
+		return ret;
+	}
+
+#endif
+
+	void TruncateFile(int fd,Uint64 size,bool quick)
 	{
 		if (FileSize(fd) == size)
 			return;
 		
+		if (quick)
+		{
 #if HAVE_FTRUNCATE64
-		if (ftruncate64(fd,size) == -1)
+			if (ftruncate64(fd,size) == -1)
 #else
-		if (ftruncate(fd,size) == -1)
+			if (ftruncate(fd,size) == -1)
+#endif	
+				throw Error(i18n("Cannot expand file : %1").arg(strerror(errno)));
+		}
+		else
+		{
+#if HAVE_POSIX_FALLOCATE64
+			if (posix_fallocate64(fd,0,size) != 0)
+				throw Error(i18n("Cannot expand file : %1").arg(strerror(errno)));
+#elif HAVE_POSIX_FALLOCATE
+			if (posix_fallocate(fd,0,size) != 0)
+				throw Error(i18n("Cannot expand file : %1").arg(strerror(errno)));
+#else
+			SeekFile(fd,0,SEEK_SET);
+			bt::Array<Uint8> buf(4096);
+			buf.fill(0);
+	
+			Uint64 written = 0;
+			while (written < size)
+			{
+				int to_write = size - written;
+				if (to_write > 4096)
+					to_write = 4096;
+				
+				int ret = write(fd,buf,to_write);
+				if (ret < 0)
+					throw Error(i18n("Cannot expand file : %1").arg(strerror(errno)));
+				else if (ret == 0 || ret != (int)to_write)
+					throw Error(i18n("Cannot expand file").arg(strerror(errno)));
+				else
+					written += to_write;
+			}
 #endif
-			throw Error(i18n("Cannot expand file : %1").arg(strerror(errno)));
+		}
 	}
 	
 	void TruncateFile(const QString & path,Uint64 size)
@@ -308,7 +406,7 @@ namespace bt
 		
 		try
 		{
-			TruncateFile(fd,size);
+			TruncateFile(fd,size,true);
 			close(fd);
 		}
 		catch (...)
@@ -328,4 +426,41 @@ namespace bt
 			throw Error(i18n("Cannot seek in file : %1").arg(strerror(errno)));
 	}
 	
+	bool FreeDiskSpace(const QString & path,Uint64 & bytes_free)
+	{
+#if HAVE_STATVFS
+#if HAVE_STATVFS64
+		struct statvfs64 stfs;
+		if (statvfs64(path.local8Bit(), &stfs) == 0)
+#else
+		struct statvfs stfs;
+		if (statvfs(path.local8Bit(), &stfs) == 0)
+#endif
+		{
+			bytes_free = ((Uint64)stfs.f_bavail) * ((Uint64)stfs.f_frsize);
+			return true;
+		}
+		else
+		{
+			Out(SYS_GEN|LOG_DEBUG) << "Error : statvfs for " << path << " failed :  "
+						<< QString(strerror(errno)) << endl;
+
+			return false;
+		}
+#else
+		struct statfs stfs;
+		if (statfs(path.local8Bit(), &stfs) == 0)
+		{
+			bytes_free = ((Uint64)stfs.f_bavail) * ((Uint64)stfs.f_bsize);
+			return true;
+		}
+		else
+		{
+			Out(SYS_GEN|LOG_DEBUG) << "Error : statfs for " << path << " failed :  "
+						<< QString(strerror(errno)) << endl;
+
+			return false;
+		}
+#endif
+	}
 }
