@@ -19,26 +19,33 @@
  ***************************************************************************/
 
 #include <kurl.h>
+#include <kprocess.h>
 #include <klocale.h>
 #include <qdatetime.h>
 #include <qtextstream.h>
 #include <qfile.h>
 #include <qptrlist.h>
 #include <iostream>
+#include <stdlib.h>
 #include <torrent/globals.h>
 #include <interfaces/logmonitorinterface.h>
 #include <qmutex.h> 
 #include <util/fileops.h>
+#include <stdlib.h>
 #include "log.h"
 #include "error.h"
+#include "autorotatelogjob.h"
 
 using namespace kt;
 
 namespace bt
 {
+	const Uint32 MAX_LOG_FILE_SIZE = 10 * 1024 * 1024; // 10 MB
+	
 	class Log::Private
 	{
 	public:
+		Log* parent;
 		QTextStream* out;
 		QFile fptr;
 		bool to_cout;
@@ -46,8 +53,9 @@ namespace bt
 		QString tmp;
 		QMutex mutex;
 		unsigned int m_filter;
+		AutoRotateLogJob* rotate_job;
 	public:
-		Private() : out(0),to_cout(false)
+		Private(Log* parent) : parent(parent),out(0),to_cout(false),rotate_job(0)
 		{
 			out = new QTextStream();
 		}
@@ -79,7 +87,7 @@ namespace bt
 			
 			// move current log to 1 and zip it
 			bt::Move(file,file + "-1",true);
-			system(QString("gzip %1-1").arg(file).local8Bit());
+			system(QString("gzip " + KProcess::quote(file + "-1")).local8Bit());
 		}
 
 		void setOutputFile(const QString & file)
@@ -101,31 +109,58 @@ namespace bt
 		{
 			tmp += line;
 		}
-
-		void endline()
+		
+		void finishLine()
 		{
-			*out << QDateTime::currentDateTime().toString() << ": " << tmp << ::endl;
-			fptr.flush();
-			if (to_cout)
-				std::cout << tmp.local8Bit() << std::endl;;
-			
-			if (monitors.count() > 0)
+			// only add stuff when we are not rotating the logs
+			// this could result in the loss of some messages
+			if (!rotate_job) 
 			{
-				QPtrList<LogMonitorInterface>::iterator i = monitors.begin();
-				while (i != monitors.end())
+				*out << QDateTime::currentDateTime().toString() << ": " << tmp << ::endl;
+				fptr.flush();
+				if (to_cout)
+					std::cout << tmp.local8Bit() << std::endl;
+				
+				if (monitors.count() > 0)
 				{
-					kt::LogMonitorInterface* lmi = *i;
-					lmi->message(tmp,m_filter);
-					i++;
+					QPtrList<LogMonitorInterface>::iterator i = monitors.begin();
+					while (i != monitors.end())
+					{
+						kt::LogMonitorInterface* lmi = *i;
+						lmi->message(tmp,m_filter);
+						i++;
+					}
 				}
 			}
 			tmp = "";
 		}
+
+		void endline()
+		{
+			finishLine();
+			if (fptr.size() > MAX_LOG_FILE_SIZE && !rotate_job)
+			{
+				tmp = "Log larger then 10 MB, rotating";
+				finishLine();
+				QString file = fptr.name();
+				fptr.close(); // close the log file
+				out->setDevice(0);		
+				// start the rotate job
+				rotate_job = new AutoRotateLogJob(file,parent); 
+			}
+		}
+		
+		void logRotateDone()
+		{
+			fptr.open(IO_WriteOnly);
+			out->setDevice(&fptr);
+			rotate_job = 0;
+		}
 	};
 	
 	Log::Log() 
 	{
-		priv = new Private();
+		priv = new Private(this);
 	}
 	
 	
@@ -200,11 +235,15 @@ namespace bt
 		priv->mutex.lock();
 	}
 	
+	void Log::logRotateDone()
+	{
+		priv->logRotateDone();
+	}
+	
 	Log & Out(unsigned int arg)
 	{
 		Log & lg = Globals::instance().getLog(arg);
 		lg.lock();
-//		lg.setOutputToConsole(Globals::instance().isDebugModeSet());
 		return lg;
 	}
 }
