@@ -74,12 +74,12 @@ namespace kt
 	{
 		m_list_view->setShowSortIndicator(true);
 		m_list_view->setAllColumnsShowFocus(true);
-		
-		
+	
 		m_list_view->setColumnAlignment(0,Qt::AlignLeft);
 		m_list_view->setColumnAlignment(1,Qt::AlignCenter);
 		m_list_view->setColumnAlignment(3,Qt::AlignRight);
 		m_list_view->setColumnAlignment(4,Qt::AlignRight);
+		curr_tc = 0;
 	}
 	
 	
@@ -109,8 +109,18 @@ namespace kt
 		items.clear();
 	}
 	
-	void ChunkDownloadView::update(kt::TorrentInterface* curr_tc)
+	void ChunkDownloadView::changeTC(kt::TorrentInterface* tc)
 	{
+		curr_tc = tc;
+		setEnabled(curr_tc != 0);
+		update();
+	}
+	
+	void ChunkDownloadView::update()
+	{
+		if (!curr_tc)
+			return;
+		
 		QMap<ChunkDownloadInterface*,ChunkDownloadViewItem*>::iterator i = items.begin();
 		while (i != items.end())
 		{
@@ -125,6 +135,7 @@ namespace kt
 		m_chunks_downloaded->setText(QString::number(s.num_chunks_downloaded));
 		m_total_chunks->setText(QString::number(s.total_chunks));
 		m_excluded_chunks->setText(QString::number(s.num_chunks_excluded));
+		m_chunks_left->setText(QString::number(s.num_chunks_left));
 		
 		if( s.chunk_size / 1024 < 1024 )
 			m_size_chunks->setText(QString::number(s.chunk_size / 1024) + "." + QString::number((s.chunk_size % 1024) / 100) + " KB");
