@@ -37,13 +37,13 @@ K_EXPORT_COMPONENT_FACTORY(ktipfilterplugin,KGenericFactory<kt::IPFilterPlugin>(
 
 namespace kt
 {	
-	const QString NAME = "ipfilterplugin";
+	const QString NAME = "IP Filter";
 	const QString AUTHOR = "Ivan Vasic";
 	const QString EMAIL = "ivasic@gmail.com";
-	const QString DESCRIPTION = i18n("IP filter plugin, filters out unwanted peers");
+	const QString DESCRIPTION = i18n("Filters out unwanted peers based on their IP address");
 
 	IPFilterPlugin::IPFilterPlugin(QObject* parent, const char* name, const QStringList& args)
-	: Plugin(parent, name, args,NAME,AUTHOR,EMAIL,DESCRIPTION)
+	: Plugin(parent, name, args,NAME,i18n("IP Filter"),AUTHOR,EMAIL,DESCRIPTION,"filter")
 	{
 		// setXMLFile("ktpluginui.rc");
 		level1 = 0;
