@@ -43,13 +43,13 @@ K_EXPORT_COMPONENT_FACTORY(ktscanfolderplugin,KGenericFactory<kt::ScanFolderPlug
 
 namespace kt
 {	
-	const QString NAME = "scanfolderplugin";
+	const QString NAME = "Scan Folder";
 	const QString AUTHOR = "Ivan Vasic";
 	const QString EMAIL = "ivasic@gmail.com";
 	const QString DESCRIPTION = i18n("Automatically scans directories for torrent files and loads them.");
 
 	ScanFolderPlugin::ScanFolderPlugin(QObject* parent, const char* name, const QStringList& args)
-	: Plugin(parent, name, args,NAME,AUTHOR,EMAIL,DESCRIPTION)
+	: Plugin(parent, name, args,NAME,i18n("Scan Folder"),AUTHOR,EMAIL,DESCRIPTION,"view_sidetree")
 	{
 // 		setXMLFile("ktscanfolderpluginui.rc");
 		m_sf1 = 0;
