@@ -50,13 +50,13 @@ K_EXPORT_COMPONENT_FACTORY(ktschedulerplugin,KGenericFactory<kt::SchedulerPlugin
 
 namespace kt
 {	
-	const QString NAME = "schedulerplugin";
+	const QString NAME = "Bandwith Scheduler";
 	const QString AUTHOR = "Ivan Vasic";
 	const QString EMAIL = "ivasic@gmail.com";
 	const QString DESCRIPTION = i18n("Bandwidth scheduling plugin");
 
 	SchedulerPlugin::SchedulerPlugin(QObject* parent, const char* name, const QStringList& args)
-	: Plugin(parent, name, args,NAME,AUTHOR,EMAIL,DESCRIPTION)
+	: Plugin(parent, name, args,NAME,i18n("Bandwidth Scheduler"),AUTHOR,EMAIL,DESCRIPTION, "clock")
 	{
 		setXMLFile("ktschedulerpluginui.rc");
 		bws_action = 0;
